@@ -19,7 +19,7 @@ use crate::request::Request;
 use crate::response::Response;
 use crate::v4::async_message::AsynchronousMessage;
 use crate::v4::interaction::V4Interaction;
-use crate::v4::sync_message::SynchronousMessages;
+use crate::v4::sync_message::SynchronousMessage;
 use crate::v4::synch_http::SynchronousHttp;
 use crate::verify_json::{json_type_of, PactFileVerificationResult, PactJsonVerifier, ResultLevel};
 
@@ -31,7 +31,7 @@ pub struct RequestResponseInteraction {
   /// Description of this interaction. This needs to be unique in the pact file.
   pub description: String,
   /// Optional provider states for the interaction.
-  /// See https://docs.pact.io/getting_started/provider_states for more info on provider states.
+  /// See `<https://docs.pact.io/getting_started/provider_states>` for more info on provider states.
   pub provider_states: Vec<ProviderState>,
   /// Request of the interaction
   pub request: Request,
@@ -68,10 +68,22 @@ impl Interaction for RequestResponseInteraction {
     self.description.clone()
   }
 
+  fn set_id(&mut self, id: Option<String>) {
+    self.id = id;
+  }
+
+  fn set_description(&mut self, description: &str) {
+    self.description = description.to_string();
+  }
+
   fn provider_states(&self) -> Vec<ProviderState> {
     self.provider_states.clone()
   }
 
+  fn provider_states_mut(&mut self) -> &mut Vec<ProviderState> {
+    &mut self.provider_states
+  }
+
   fn contents(&self) -> OptionalBody {
     self.response.body.clone()
   }
@@ -92,6 +104,10 @@ impl Interaction for RequestResponseInteraction {
     self.as_v4_http().map(|i| i.boxed_v4())
   }
 
+  fn as_v4_mut(&mut self) -> Option<&mut dyn V4Interaction> {
+    None
+  }
+
   fn as_v4_http(&self) -> Option<SynchronousHttp> {
     Some(SynchronousHttp {
       id: self.id.clone(),
@@ -108,15 +124,27 @@ impl Interaction for RequestResponseInteraction {
     None
   }
 
-  fn as_v4_sync_message(&self) -> Option<SynchronousMessages> {
+  fn as_v4_sync_message(&self) -> Option<SynchronousMessage> {
     None
   }
 
-  fn boxed(&self) -> Box<dyn Interaction + Send> {
+  fn as_v4_http_mut(&mut self) -> Option<&mut SynchronousHttp> {
+    None
+  }
+
+  fn as_v4_async_message_mut(&mut self) -> Option<&mut AsynchronousMessage> {
+    None
+  }
+
+  fn as_v4_sync_message_mut(&mut self) -> Option<&mut SynchronousMessage> {
+    None
+  }
+
+  fn boxed(&self) -> Box<dyn Interaction + Send + Sync> {
     Box::new(self.clone())
   }
 
-  fn arced(&self) -> Arc<dyn Interaction + Send> {
+  fn arced(&self) -> Arc<dyn Interaction + Send + Sync> {
     Arc::new(self.clone())
   }
 
