@@ -1,17 +1,20 @@
 //! V4 specification models - HTTP parts for SynchronousHttp
 
 use std::collections::HashMap;
+use std::convert::TryFrom;
 use std::fmt::{Display, Formatter};
 use std::hash::{Hash, Hasher};
 
 use base64::decode;
 use bytes::BytesMut;
 use log::*;
+use maplit::*;
 use serde_json::{json, Value};
 
 use crate::bodies::OptionalBody;
-use crate::content_types::{ContentType, detect_content_type_from_bytes};
+use crate::content_types::{ContentType, ContentTypeHint, detect_content_type_from_bytes};
 use crate::generators::{Generators, generators_from_json, generators_to_json};
+use crate::http_parts::HttpPart;
 use crate::json_utils::{headers_from_json, json_to_string};
 use crate::matchingrules::{matchers_from_json, matchers_to_json, MatchingRules};
 use crate::PactSpecification;
@@ -162,6 +165,47 @@ impl Hash for HttpRequest {
   }
 }
 
+impl HttpPart for HttpRequest {
+  fn headers(&self) -> &Option<HashMap<String, Vec<String>>> {
+    &self.headers
+  }
+
+  fn headers_mut(&mut self) -> &mut HashMap<String, Vec<String>> {
+    if self.headers.is_none() {
+      self.headers = Some(hashmap!{});
+    }
+    self.headers.as_mut().unwrap()
+  }
+
+  fn body(&self) -> &OptionalBody {
+    &self.body
+  }
+
+  fn body_mut(&mut self) -> &mut OptionalBody {
+    &mut self.body
+  }
+
+  fn matching_rules(&self) -> &MatchingRules {
+    &self.matching_rules
+  }
+
+  fn matching_rules_mut(&mut self) -> &mut MatchingRules {
+    &mut self.matching_rules
+  }
+
+  fn generators(&self) -> &Generators {
+    &self.generators
+  }
+
+  fn generators_mut(&mut self) -> &mut Generators {
+    &mut self.generators
+  }
+
+  fn lookup_content_type(&self) -> Option<String> {
+    self.lookup_header_value("content-type")
+  }
+}
+
 pub fn body_from_json(json: &Value, attr_name: &str, headers: &Option<HashMap<String, Vec<String>>>) -> OptionalBody {
   match json.get(attr_name) {
     Some(body) => match *body {
@@ -208,6 +252,23 @@ pub fn body_from_json(json: &Value, attr_name: &str, headers: &Option<HashMap<St
               None => (false, Default::default())
             };
 
+            let ct_override = body_attrs.get("contentTypeHint")
+              .map(|val| {
+                match val {
+                  Value::String(s) => match ContentTypeHint::try_from(s.as_str()) {
+                    Ok(val) => val,
+                    Err(err) => {
+                      warn!("'{}' is not a valid value for contentTypeHint, ignoring - {}", s, err);
+                      ContentTypeHint::DEFAULT
+                    }
+                  }
+                  _ => {
+                    warn!("'{}' is not a valid value for contentTypeHint, ignoring", val);
+                    ContentTypeHint::DEFAULT
+                  }
+                }
+              });
+
             let body_bytes = if encoded {
               match encoding.as_str() {
                 "base64" => {
@@ -237,7 +298,7 @@ pub fn body_from_json(json: &Value, attr_name: &str, headers: &Option<HashMap<St
               });
               let mut buf = BytesMut::new();
               buf.extend_from_slice(&*body_bytes);
-              OptionalBody::Present(buf.freeze(), Some(content_type))
+              OptionalBody::Present(buf.freeze(), Some(content_type), ct_override)
             }
           },
           None => OptionalBody::Missing
@@ -246,7 +307,7 @@ pub fn body_from_json(json: &Value, attr_name: &str, headers: &Option<HashMap<St
       Value::Null => OptionalBody::Null,
       _ => {
         warn!("Body in attribute '{}' from JSON file is not formatted correctly, will load it as plain text", attr_name);
-        OptionalBody::Present(body.to_string().into(), None)
+        OptionalBody::Present(body.to_string().into(), None, None)
       }
     },
     None => OptionalBody::Missing
@@ -397,6 +458,52 @@ impl HttpResponse {
   pub fn content_type(&self) -> Option<ContentType> {
     calc_content_type(&self.body, &self.headers)
   }
+
+  /// If this response represents a success (status code < 400)
+  pub fn is_success(&self) -> bool {
+    self.status < 400
+  }
+}
+
+impl HttpPart for HttpResponse {
+  fn headers(&self) -> &Option<HashMap<String, Vec<String>>> {
+    &self.headers
+  }
+
+  fn headers_mut(&mut self) -> &mut HashMap<String, Vec<String>> {
+    if self.headers.is_none() {
+      self.headers = Some(hashmap!{});
+    }
+    self.headers.as_mut().unwrap()
+  }
+
+  fn body(&self) -> &OptionalBody {
+    &self.body
+  }
+
+  fn body_mut(&mut self) -> &mut OptionalBody {
+    &mut self.body
+  }
+
+  fn matching_rules(&self) -> &MatchingRules {
+    &self.matching_rules
+  }
+
+  fn matching_rules_mut(&mut self) -> &mut MatchingRules {
+    &mut self.matching_rules
+  }
+
+  fn generators(&self) -> &Generators {
+    &self.generators
+  }
+
+  fn generators_mut(&mut self) -> &mut Generators {
+    &mut self.generators
+  }
+
+  fn lookup_content_type(&self) -> Option<String> {
+    self.lookup_header_value("content-type")
+  }
 }
 
 #[cfg(test)]
@@ -409,7 +516,7 @@ mod tests {
   use serde_json::json;
 
   use crate::bodies::OptionalBody;
-  use crate::content_types::JSON;
+  use crate::content_types::{JSON, ContentTypeHint};
   use crate::json_utils::headers_from_json;
   use crate::v4::http_parts::{body_from_json, HttpRequest, HttpResponse};
 
@@ -540,9 +647,13 @@ mod tests {
 
   #[test]
   fn http_request_to_json_with_json_body() {
-    let request = HttpRequest { headers: Some(hashmap!{
-    "Content-Type".to_string() => vec!["application/json".to_string()]
-  }), body: OptionalBody::Present(r#"{"key": "value"}"#.into(), Some("application/json".into())), .. HttpRequest::default() };
+    let request = HttpRequest {
+      headers: Some(hashmap! {
+        "Content-Type".to_string() => vec!["application/json".to_string()]
+      }),
+      body: OptionalBody::Present(r#"{"key": "value"}"#.into(), Some("application/json".into()), None),
+      ..HttpRequest::default()
+    };
     expect!(request.to_json().to_string()).to(
       be_equal_to(r#"{"body":{"content":{"key":"value"},"contentType":"application/json","encoded":false},"headers":{"Content-Type":["application/json"]},"method":"GET","path":"/"}"#)
     );
@@ -550,8 +661,11 @@ mod tests {
 
   #[test]
   fn http_request_to_json_with_non_json_body() {
-    let request = HttpRequest { headers: Some(hashmap!{ "Content-Type".to_string() => vec!["text/plain".to_string()] }),
-      body: OptionalBody::Present("This is some text".into(), Some("text/plain".into())), .. HttpRequest::default() };
+    let request = HttpRequest {
+      headers: Some(hashmap! { "Content-Type".to_string() => vec!["text/plain".to_string()] }),
+      body: OptionalBody::Present("This is some text".into(), Some("text/plain".into()), None),
+      ..HttpRequest::default()
+    };
     expect!(request.to_json().to_string()).to(
       be_equal_to(r#"{"body":{"content":"This is some text","contentType":"text/plain","encoded":false},"headers":{"Content-Type":["text/plain"]},"method":"GET","path":"/"}"#)
     );
@@ -592,9 +706,13 @@ mod tests {
 
   #[test]
   fn http_response_to_json_with_json_body() {
-    let response = HttpResponse { headers: Some(hashmap!{
+    let response = HttpResponse {
+      headers: Some(hashmap! {
         "Content-Type".to_string() => vec!["application/json".to_string()]
-    }), body: OptionalBody::Present(r#"{"key": "value"}"#.into(), Some("application/json".into())), .. HttpResponse::default() };
+    }),
+      body: OptionalBody::Present(r#"{"key": "value"}"#.into(), Some("application/json".into()), None),
+      ..HttpResponse::default()
+    };
     expect!(response.to_json().to_string()).to(
       be_equal_to(r#"{"body":{"content":{"key":"value"},"contentType":"application/json","encoded":false},"headers":{"Content-Type":["application/json"]},"status":200}"#)
     );
@@ -602,8 +720,11 @@ mod tests {
 
   #[test]
   fn http_response_to_json_with_non_json_body() {
-    let response = HttpResponse { headers: Some(hashmap!{ "Content-Type".to_string() => vec!["text/plain".to_string()] }),
-      body: OptionalBody::Present("This is some text".into(), "text/plain".parse().ok()), .. HttpResponse::default() };
+    let response = HttpResponse {
+      headers: Some(hashmap! { "Content-Type".to_string() => vec!["text/plain".to_string()] }),
+      body: OptionalBody::Present("This is some text".into(), "text/plain".parse().ok(), None),
+      ..HttpResponse::default()
+    };
     expect!(response.to_json().to_string()).to(
       be_equal_to(r#"{"body":{"content":"This is some text","contentType":"text/plain","encoded":false},"headers":{"Content-Type":["text/plain"]},"status":200}"#)
     );
@@ -696,7 +817,7 @@ mod tests {
     });
     let body = body_from_json(&json, "body", &None);
     expect!(body).to(be_equal_to(OptionalBody::Present("{\"test\":true}".into(),
-                                                       Some(JSON.clone()))));
+                                                       Some(JSON.clone()), None)));
   }
 
   #[test]
@@ -726,7 +847,7 @@ mod tests {
     let body = body_from_json(&json, "body", &None);
     expect!(body).to(be_equal_to(
       OptionalBody::Present("<?xml version=\"1.0\"?> <body></body>".into(),
-                            Some("application/xml".into()))));
+                            Some("application/xml".into()), None)));
   }
 
   #[test]
@@ -741,7 +862,7 @@ mod tests {
     });
     let headers = headers_from_json(&json);
     let body = body_from_json(&json, "body", &headers);
-    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is a string\"".into(), Some("text/plain".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is a string\"".into(), Some("text/plain".into()), None)));
   }
 
   #[test]
@@ -756,7 +877,7 @@ mod tests {
     });
     let headers = headers_from_json(&json);
     let body = body_from_json(&json, "body", &headers);
-    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is a string\"".into(), Some("text/html".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is a string\"".into(), Some("text/html".into()), None)));
   }
 
   #[test]
@@ -769,7 +890,7 @@ mod tests {
       }
     });
     let body = body_from_json(&json, "body", &None);
-    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is actually a JSON string\"".into(), Some("application/json".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("\"This is actually a JSON string\"".into(), Some("application/json".into()), None)));
   }
 
   #[test]
@@ -784,7 +905,24 @@ mod tests {
     });
     let headers = headers_from_json(&json);
     let body = body_from_json(&json, "body", &headers);
-    expect!(body).to(be_equal_to(OptionalBody::Present("{\"test\":true}".into(), Some("application/json".into()))));
+    expect!(body).to(be_equal_to(OptionalBody::Present("{\"test\":true}".into(), Some("application/json".into()), None)));
   }
 
+  #[test]
+  fn body_with_an_overridden_content_type_format() {
+    let json = json!({
+      "body": {
+        "content": "Cg9wYWN0LWp2bS1kcml2ZXISBTAuMC4w",
+        "contentType": "application/stuff",
+        "contentTypeHint": "BINARY",
+        "encoded": "base64"
+      }
+    });
+    let body = body_from_json(&json, "body", &None);
+    expect!(body).to(be_equal_to(
+      OptionalBody::Present(
+        "\npact-jvm-driver0.0.0".into(),
+        Some("application/stuff".into()),
+        Some(ContentTypeHint::BINARY))));
+  }
 }
